# -*- coding: utf-8 -*-
#
# ApplicationIntegrationServer - IMAP/CSV Example Plugin
#                                Read user base from a CSV file and perform 
#                                authentication against an IMAP server
#
# Copyright (c) 2014 - 2017 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import logging
import imaplib
import os.path
import csv


class Plugin:
    def __init__(self):
        csv_file_name = "exIMAP.csv"
        self.csv_file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), csv_file_name)
        self.csv_fieldnames = ("userName", "emailAddresses")
        self.imap_port = 993
        self.imap_host = "imap.example.com"
        self.logger = logging.getLogger("AppIntLogger")
        # credentials to use this plugin
        self.plugin_user = "admin"
        self.plugin_password = "password"

    def list_(self, _):
        user_list = list()
        with open(self.csv_file_path, "r") as csv_file:
            userfile = csv.DictReader(csv_file, self.csv_fieldnames, delimiter=";")
            for line in userfile:
                user_list.append({"userName": line["userName"],
                                  "emailAddresses": line["emailAddresses"].split(",")})
        return user_list

    def auth(self, username, password):
        imap_connection = imaplib.IMAP4_SSL(self.imap_host, self.imap_port)
        try:
            return_value = imap_connection.login(username, password)[0].lower() == "ok"
        except Exception as e:
            self.logger.error("PLUGIN ERROR: {}".format(e))
            return {"succeeded": False}
        else:
            return {"succeeded": return_value}

    def http_authentication(self, username, password):
        return username == self.plugin_user and password == self.plugin_password
